@@ -31,7 +31,8 @@ typedef struct {
 } cmark_code;
 
 typedef struct {
-  int level;
+  int internal_offset;
+  int8_t level;
   bool setext;
 } cmark_heading;
 
@@ -69,7 +70,6 @@ struct cmark_node {
   int start_column;
   int end_line;
   int end_column;
-  int internal_offset;
   uint16_t type;
   uint16_t flags;
 
